import os
from mistune import create_markdown
from mistune.directives import (
    RSTDirective,
    FencedDirective,
    Admonition,
    TableOfContents,
    Include,
    Image,
    Figure,
)
from tests import BaseTestCase
from tests.fixtures import ROOT


def load_directive_test(filename, directive, cls):
    class TestDirective(BaseTestCase):
        @staticmethod
        def parse(text):
            md = create_markdown(
                escape=False,
                plugins=[cls([directive])],
            )
            html = md(text)
            return html

    TestDirective.load_fixtures(filename + '.txt')
    globals()["TestDirective_" + filename] = TestDirective


load_directive_test('rst_admonition', Admonition(), RSTDirective)
load_directive_test('rst_toc', TableOfContents(), RSTDirective)
load_directive_test('fenced_admonition', Admonition(), FencedDirective)
load_directive_test('fenced_toc', TableOfContents(), FencedDirective)
load_directive_test('fenced_image', Image(), FencedDirective)
load_directive_test('fenced_figure', Figure(), FencedDirective)


class CustomizeTableOfContents(TableOfContents):
    def generate_heading_id(self, token, i):
        return 't-' + str(i + 1)


class TestCustomizeToc(BaseTestCase):
    def test_rst_toc(self):
        md = create_markdown(
            escape=False,
            plugins=[
                RSTDirective([CustomizeTableOfContents()]),
            ],
        )
        html = md('# h1\n\n.. toc::\n')
        self.assertIn('<h1 id="t-1">h1</h1>', html)
        self.assertIn('<a href="#t-1">h1</a>', html)

    def test_fenced_toc(self):
        md = create_markdown(
            escape=False,
            plugins=[
                FencedDirective([CustomizeTableOfContents()]),
            ],
        )
        html = md('# h1\n\n```{toc}\n```\n')
        self.assertIn('<h1 id="t-1">h1</h1>', html)
        self.assertIn('<a href="#t-1">h1</a>', html)

    def test_colon_fenced_toc(self):
        md = create_markdown(
            escape=False,
            plugins=[
                FencedDirective([CustomizeTableOfContents()], ':'),
            ],
        )
        html = md('# h1\n\n:::{toc}\n:::\n')
        self.assertIn('<h1 id="t-1">h1</h1>', html)
        self.assertIn('<a href="#t-1">h1</a>', html)


class TestDirectiveInclude(BaseTestCase):
    md = create_markdown(escape=False, plugins=[RSTDirective([Include()])])

    def test_html_include(self):
        html = self.md.read(os.path.join(ROOT, 'include/text.md'))[0]
        self.assertIn('Could not include self', html)
        self.assertIn('Could not find file', html)
        self.assertIn('<div>include html</div>', html)
        self.assertIn('<blockquote>', html)
        self.assertIn('# Table of Contents', html)

    def test_include_missing_source(self):
        s = '.. include:: foo.txt'
        html = self.md(s)
        self.assertIn('Missing source file', html)



import os
from mistune import create_markdown
from mistune.toc import add_toc_hook, render_toc_ul
from tests import BaseTestCase


class TestTocHook(BaseTestCase):
    @staticmethod
    def parse(text):
        md = create_markdown(escape=False)
        add_toc_hook(md)
        html, state = md.parse(text)
        result = html + render_toc_ul(state.env['toc_items'])
        return result

    def test_customize_heading_id_func(self):
        def heading_id(token, i):
            return 't-' + str(i + 1)

        md = create_markdown(escape=False)
        add_toc_hook(md, heading_id=heading_id)
        html = md('# h1')
        self.assertEqual(html, '<h1 id="t-1">h1</h1>\n')


TestTocHook.load_fixtures('hook_toc.txt')



from mistune import create_markdown
from mistune.plugins.table import table_in_list, table_in_quote
from mistune.plugins.math import math_in_list, math_in_quote
from tests import BaseTestCase


def load_plugin(plugin_name):
    md1 = create_markdown(escape=False, plugins=[plugin_name])
    md2 = create_markdown(escape=False, plugins=[plugin_name, 'speedup'])

    class TestPlugin1(BaseTestCase):
        parse = md1

    class TestPlugin2(BaseTestCase):
        parse = md2

    TestPlugin1.load_fixtures(plugin_name + ".txt")
    TestPlugin2.load_fixtures(plugin_name + ".txt")
    globals()["TestPlugin1_" + plugin_name] = TestPlugin1
    globals()["TestPlugin2_" + plugin_name] = TestPlugin2


load_plugin("url")
load_plugin("strikethrough")
load_plugin("mark")
load_plugin("insert")
load_plugin("superscript")
load_plugin("subscript")
load_plugin("task_lists")
load_plugin("table")
load_plugin("def_list")
load_plugin("footnotes")
load_plugin("abbr")
load_plugin("math")
load_plugin("ruby")
load_plugin("spoiler")


class TestExtraPlugins(BaseTestCase):
    def test_table_in_list(self):
        text = '''- Cell | Cell\n  ---- | ----\n   1  |  2\n'''
        md1 = create_markdown(escape=False)
        md2 = create_markdown(escape=False, plugins=['table', table_in_list])
        self.assertNotIn('<table>', md1(text))
        self.assertIn('<table>', md2(text))

    def test_table_in_quote(self):
        text = '''> Cell | Cell\n> ---- | ----\n>  1  |  2\n'''
        md1 = create_markdown(escape=False)
        md2 = create_markdown(escape=False, plugins=['table', table_in_quote])
        self.assertNotIn('<table>', md1(text))
        self.assertIn('<table>', md2(text))

    def test_math_in_list(self):
        text = '''- $$\n  foo\n  $$\n'''
        md1 = create_markdown(escape=False)
        md2 = create_markdown(escape=False, plugins=['math', math_in_list])
        self.assertNotIn('class="math"', md1(text))
        self.assertIn('class="math"', md2(text))

    def test_math_in_quote(self):
        text = '''> $$\n> foo\n> $$\n'''
        md1 = create_markdown(escape=False)
        md2 = create_markdown(escape=False, plugins=['math', math_in_quote])
        self.assertNotIn('class="math"', md1(text))
        self.assertIn('class="math"', md2(text))



import mistune
from unittest import TestCase


class TestMiscCases(TestCase):
    def test_none(self):
        self.assertEqual(mistune.html(None), '')

    def test_before_parse_hooks(self):
        def _add_name(md, state):
            state.env['name'] = 'test'

        md = mistune.create_markdown()
        md.before_parse_hooks.append(_add_name)
        state = md.block.state_cls()
        md.parse('', state)
        self.assertEqual(state.env['name'], 'test')

    def test_hard_wrap(self):
        md = mistune.create_markdown(escape=False, hard_wrap=True)
        result = md('foo\nbar')
        expected = '<p>foo<br />\nbar</p>'
        self.assertEqual(result.strip(), expected)

        md = mistune.create_markdown(
            escape=False, hard_wrap=True, plugins=['speedup'])
        result = md('foo\nbar')
        self.assertEqual(result.strip(), expected)

    def test_escape_html(self):
        md = mistune.create_markdown(escape=True)
        result = md('<div>1</div>')
        expected = '<p>&lt;div&gt;1&lt;/div&gt;</p>'
        self.assertEqual(result.strip(), expected)

        result = md('<em>1</em>')
        expected = '<p>&lt;em&gt;1&lt;/em&gt;</p>'
        self.assertEqual(result.strip(), expected)

    def test_harmful_links(self):
        result = mistune.html('[h](javAscript:alert)')
        expected = '<p><a href="#harmful-link">h</a></p>'
        self.assertEqual(result.strip(), expected)

    def test_allow_harmful_protocols(self):
        renderer = mistune.HTMLRenderer(allow_harmful_protocols=True)
        md = mistune.Markdown(renderer)
        result = md('[h](javascript:alert)')
        expected = '<p><a href="javascript:alert">h</a></p>'
        self.assertEqual(result.strip(), expected)

    def test_allow_data_protocols(self):
        renderer = mistune.HTMLRenderer(allow_harmful_protocols=['data:'])
        md = mistune.Markdown(renderer)
        result = md('[h](data:alert)')
        expected = '<p><a href="data:alert">h</a></p>'
        self.assertEqual(result.strip(), expected)

    def test_use_plugin(self):
        from mistune.plugins.url import url
        md = mistune.Markdown(mistune.HTMLRenderer())
        md.use(url)

    def test_markdown_func(self):
        result = mistune.markdown('**b**')
        expected = '<p><strong>b</strong></p>\n'
        self.assertEqual(result, expected)

        # trigger to use cached parser
        result = mistune.markdown('**b**')
        self.assertEqual(result, expected)

    def test_ast_output(self):
        md = mistune.create_markdown(escape=False, renderer=None)
        text = '# h1\n\nfoo **bar**'
        result = md(text)
        expected = [
            {
                'type': 'heading',
                'children': [{'type': 'text', 'raw': 'h1'}],
                'attrs': {'level': 1},
                'style': 'axt',
            },
            {'type': 'blank_line'},
            {
                'type': 'paragraph',
                'children': [
                    {'type': 'text', 'raw': 'foo '},
                    {'type': 'strong', 'children': [{'type': 'text', 'raw': 'bar'}]}
                ]
            },
        ]
        self.assertEqual(result, expected)



from mistune import create_markdown
from mistune.renderers.rst import RSTRenderer
from mistune.renderers.markdown import MarkdownRenderer
from tests import BaseTestCase


def load_renderer(renderer):
    class TestRenderer(BaseTestCase):
        parse = create_markdown(renderer=renderer)

    name = renderer.NAME
    TestRenderer.load_fixtures('renderer_' + name + '.txt')
    globals()["TestRenderer" + name.title()] = TestRenderer


load_renderer(RSTRenderer())
load_renderer(MarkdownRenderer())



import mistune
from tests import BaseTestCase, normalize_html


DIFF_CASES = {
    'setext_headings_093',
    'html_blocks_191',  # mistune keeps \n

    'images_573', # image can not be in image

    'links_495',
    'links_517',  # aggressive link group
    'links_518',
    'links_519',
    'links_531',
    'links_532',
}

IGNORE_CASES = {
    # we don't support link title in (title)
    'links_496',
    'links_504',

    # we don't support flanking delimiter run
    'emphasis_and_strong_emphasis_352',
    'emphasis_and_strong_emphasis_367',
    'emphasis_and_strong_emphasis_368',
    'emphasis_and_strong_emphasis_372',
    'emphasis_and_strong_emphasis_379',
    'emphasis_and_strong_emphasis_388',
    'emphasis_and_strong_emphasis_391',

    'emphasis_and_strong_emphasis_406',
    'emphasis_and_strong_emphasis_407',
    'emphasis_and_strong_emphasis_408',

    'emphasis_and_strong_emphasis_412',
    'emphasis_and_strong_emphasis_413',
    'emphasis_and_strong_emphasis_414',
    'emphasis_and_strong_emphasis_416',
    'emphasis_and_strong_emphasis_417',
    'emphasis_and_strong_emphasis_418',

    'emphasis_and_strong_emphasis_424',
    'emphasis_and_strong_emphasis_425',
    'emphasis_and_strong_emphasis_426',
    'emphasis_and_strong_emphasis_429',
    'emphasis_and_strong_emphasis_430',
    'emphasis_and_strong_emphasis_431',
    'emphasis_and_strong_emphasis_460',
    'emphasis_and_strong_emphasis_467',
    'emphasis_and_strong_emphasis_470',
    'emphasis_and_strong_emphasis_471',
    'emphasis_and_strong_emphasis_477',
    'emphasis_and_strong_emphasis_478',
}

for i in range(441, 447):
    IGNORE_CASES.add('emphasis_and_strong_emphasis_' + str(i))
for i in range(453, 459):
    IGNORE_CASES.add('emphasis_and_strong_emphasis_' + str(i))
for i in range(462, 466):
    IGNORE_CASES.add('emphasis_and_strong_emphasis_' + str(i))


class TestCommonMark(BaseTestCase):
    @classmethod
    def ignore_case(cls, n):
        return n in IGNORE_CASES or n in DIFF_CASES

    def assert_case(self, n, text, html):
        result = mistune.html(text)
        self.assertEqual(normalize_html(result), normalize_html(html))


TestCommonMark.load_fixtures('commonmark.json')



import mistune
from tests import BaseTestCase, normalize_html


class TestSyntax(BaseTestCase):
    def assert_case(self, n, text, html):
        result = mistune.html(text)
        self.assertEqual(normalize_html(result), normalize_html(html))


TestSyntax.load_fixtures('fix-commonmark.txt')
TestSyntax.load_fixtures('diff-commonmark.txt')
